<?php
namespace Ep\Wp;

use Ak\Wp\Plugin\PluginLoader;
use Ep\Middleware\BaseUrl;
use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface;
use Psr\Http\Server\MiddlewareInterface;
use Psr\Http\Server\RequestHandlerInterface;
use Zend\Db\Adapter\AdapterInterface;
use Zend\Diactoros\Response\HtmlResponse;
use Zend\Diactoros\Response\RedirectResponse;

/**
 * WordPress middleware
 *
 * @package Ep\Wp
 * @author Elektronika Perhoč <web@ep.hr>
 * @link http://gitlab.ep.local/ep-framework/wordpress
 */
class WordPress implements MiddlewareInterface
{
    /** @var array */
    protected $config;
    /** @var AdapterInterface */
    protected $db;
    /** @var PluginLoader */
    protected $pluginLoader;
    /** @var bool  */
    private $init = false;

    /**
     * WordPress constructor.
     *
     * @param array $config
     * @param AdapterInterface $db
     * @param PluginLoader $pluginLoader
     */
    public function __construct(array $config, AdapterInterface $db, PluginLoader $pluginLoader)
    {
        require_once __DIR__ . '/_functions.php';
        require_once ABSPATH . 'wp-includes/pluggable.php';

        $this->config        = $config;
        $this->db            = $db;
        $this->pluginLoader = $pluginLoader;
    }

    public function init(string $baseUrl = '/') {
        if (!$this->init) {
            define('WP_DEBUG', $this->config['debug'] ?? false);
            define('WP_DEBUG_DISPLAY', WP_DEBUG);

            $this->pluginLoader->register();

            global $wpdb;
            $wpdb = new Core\Db($this->db);

            $this->paths($baseUrl);
            $this->globals();
            $this->includes();

            $this->init = true;
        }
    }

    /**
     * Process an incoming server request and return a response, optionally delegating
     * response creation to a handler.
     *
     * @param ServerRequestInterface $request
     * @param RequestHandlerInterface $handler
     *
     * @return ResponseInterface
     */
    public function process(ServerRequestInterface $request, RequestHandlerInterface $handler): ResponseInterface
    {
        $this->init($request->getAttribute(BaseUrl::BASE_URL, '/'));

        if (defined('WP_INSTALLING')) {
            switch (intval($_GET['step'] ?? '')) {
                case 0:
                    require_once ABSPATH . 'wp-admin/includes/file.php';
                    require_once ABSPATH . 'wp-admin/includes/translation-install.php';

                    wp_download_language_pack('hr');

                    $uri = $request->getUri();
                    return new RedirectResponse(
                        $uri->withPath($request->getAttribute(BaseUrl::BASE_URL) . $uri->getPath())
                            ->withQuery('language=hr&step=1')
                    );
                case 1:
                    $_POST['weblog_title'] = $this->config['name'] ?? 'EP WordPress';
                    $_POST['user_name'] = 'Elektronika Perhoc';
                    $_POST['admin_password'] = 'Elektr0nik@Perh0c';
                    $_POST['admin_password2'] = 'Elektr0nik@Perh0c';
                    $_POST['admin_email'] = $this->config['email'] ?? 'web@ep.hr';
                    $_POST['blog_public'] = 1;
                    $_GET['step'] = 2;
                    break;
            }
        }

        if (defined('DOING_AJAX')) {
            error_reporting(0);
        }

        // Ako dopuštamo WordPressu da nastavi sa loadanjem vraćamo prazan response.
        if (defined('DOING_CRON') || defined('WP_ADMIN') || defined('WP_INSTALLING') || preg_match('#-login.php$#', $request->getUri()->getPath())) {
            return new HtmlResponse('');
        }

        if (!defined('WP_USE_THEMES')) {
            define('WP_USE_THEMES', true);
        }

        wp();

        ob_start();
        require_once ABSPATH . WPINC . '/template-loader.php';
        return new HtmlResponse(ob_get_clean());
    }

    /**
     * Postavlja putanje i url-ove.
     *
     * @param string $baseUrl
     */
    private function paths(string $baseUrl)
    {
        $baseUrl = preg_replace('#/wp(/wp-admin)?/?#', '', $baseUrl);

        $home = $baseUrl;

        define('WP_HOME', 'https://' . $_SERVER['HTTP_HOST'] . rtrim($home, '/'));
        define('WP_SITEURL', WP_HOME . '/wp');

        define('WP_CONTENT_DIR', dirname(ABSPATH) . '/wp-content');
        define('WP_CONTENT_URL', WP_HOME . '/wp-content');
    }

    /**
     * Inicijalizira globalne varijable.
     */
    private function globals()
    {
        define('WP_DEFAULT_THEME', 'flatsome');

        foreach ($this->config['auth_keys'] as $name => $val) {
            define(strtoupper($name), $val);
        }
    }

    /**
     * Učitava WordPress.
     */
    private function includes()
    {
        /** @noinspection PhpUnusedLocalVariableInspection */
        $table_prefix = 'wp_';

        /** @noinspection PhpUnusedLocalVariableInspection */
        global $wc_cpdf, $extensions_uri, $nav_elements, $extensions_url,
               $revSliderVersion, $timeline_express_base;

        require ABSPATH . 'wp-settings.php';
    }
}